#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests
import time
import datetime
import subprocess
import json
import subprocess
from subprocess import Popen


# Myブロックチェーン API
api_host = "192.168.11.101"
api_port = "3000"
url = "http://"+api_host+":"+api_port

# 処理間隔
interval = 60

# 10分前の時間を計算するための値
m10 = datetime.timedelta(minutes=10)

# 条件の設定
temperature_to_start = 29
humidity_to_start = 55
temperature_to_stop = 26
humidity_to_stop = 50


# コマンド実行関数
def do_command(cmd):
	try:
		p = Popen(cmd, shell=True)
		return p.wait()
	except Exception, e:
		print "do_command(%s) failed:%s" % (cmd, e)
		return 1


# ファンを回す
def start_fan():
	print "Start FAN"
	return do_command("/home/pi/CQ/FAN/hub-ctrl -h 0 -P 2 -p 1")


# ファンを止める
def stop_fan():
	print "Stop FAN"
	return do_command("/home/pi/CQ/FAN/hub-ctrl -h 0 -P 2 -p 0")


# Myブロックチェーンのブロックからデータを取得する
def get_block(id):
	data = None
	try:
		# ブロックを取得
		resp = requests.get(url+"/block/" + str(id))

		# 正常終了？
		if resp.status_code == 200:
			# dataを取得
			cont = resp.json()['data']

			# オブジェクトに変換
			try:
				data = json.loads(cont)
			except:
				data = []
	except Exception, e:
		print e

        return data


# ファンの操作を記録する
def save_log(msg):
        savedata = {"data":msg}
        out = json.JSONEncoder().encode(savedata)
        resp = requests.post(url+"/block/", data=out, headers={'content-type': 'application/json'})


# メインループ
fan_is_active = False
cur_block = 1
while True:
	# ブロック取得
	block = get_block(cur_block)

	# ブロックが取得できたら中を見る
	if block is not None:
		if len(block) > 0:
			# 全部ではなく最後のデータだけ見る
			b = block[len(block)-1]

			# 現在時
			now = now = datetime.datetime.now()

			# データ取得日時
			t = datetime.datetime.strptime(b['time'], "%Y-%m-%dT%H:%M:%S.%f")

			# 10分以上前のデータは無視
			if now - m10 < t:
				temperature = b['temperature']
				humidity = b['humidity']
				print "check condition:%d,%d" % (temperature, humidity)
				if fan_is_active == True:
					# 動作中なら停止条件を確認
					if temperature_to_stop > temperature or humidity_to_stop > humidity:
						if stop_fan() == 0:
							save_log("stop FAN at " + now.isoformat())
                                                        fan_is_active = False
				else:
					# 動作中でなければ開始条件を確認
					if temperature_to_start < temperature or humidity_to_start < humidity:
						if start_fan() == 0:
							save_log("start FAN at " + now.isoformat())
                                                        fan_is_active = True

		cur_block = cur_block + 1
	else:
	       	# ブロックが取得できなければ少し待つ
	        time.sleep(interval)

